/* TP 3 de LIFASR5, N. Louvet & L. Gonnord*/
/* Expérimentations autour de la non fermeture de descripteur */

#include <iostream>
#include <cstring>

#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>

// Signal handling
#include <signal.h> 

#include "rw-int.h"

using namespace std;


// Messages d'erreur
void die(int e = 1, const string &msg = "") {
  if(msg != "") {
    cerr << "Erreur : " << msg << endl;
    cerr << strerror(errno) << endl;
  }
  else cerr << "Erreur : " << strerror(errno) << endl;
  exit(e);
}

// Gestionnaire du signal SIGPIPE
void gest_sigpipe(int s){
  cout << "signal "  << s << " reçu par le processus " << getpid() << endl; 
}

int main(void) {
  int pid, p[2];

  if( pipe(p) == -1 ) die();
  if( (pid = fork()) == -1) die();

  // Mise en place d'un gestionnaire pour le signal SIGPIPE.
  struct sigaction act;
  // On récupére la structure sigaction actuelle pour SIGPIPE.
  if( sigaction(SIGPIPE, NULL, &act) == -1 ) die();
  // On remplace dans act le gestionnaire de signal par le notre.
  act.sa_handler = &gest_sigpipe;
  // On installe le nouveau gestionnaire de signal.
  if( sigaction(SIGPIPE, &act, NULL) == -1 ) die();
    
  if(pid > 0) { // processus père
    cout << "(père) mon PID est "<< getpid() << endl << flush;
    cout << "(père) le PID de mon fils est "<< pid << endl << flush;
    close(p[0]);
    int wr;
    for (int i = 1; i<100; i++){
      wr = write_int(p[1], i);
      if(wr == -1) die();
      cout << "(père) " << i << endl << flush;
    }
    close(p[1]); 
    if( waitpid(pid, NULL, 0) == -1) die();
    cout << "(père) terminaison normale." << endl << flush;
  }
  else { // processus fils
    cout << "      (fils) mon PID est " << pid << endl << flush;
    close(p[1]); 

    // Lecture d'un seul entier sur le pipe
    int n;
    if( read_int(p[0], n) == -1 ) die();
    cout << "      (fils) j'ai lu " << n << endl << flush;

    // Le fils ferme tout de suite son descripteur de fichier en lecture,
    // alors que le père écrit encore de l'autre côté !
    close(p[0]);
    cout << "      (fils) terminaison normale." << endl << flush;
  }
  return 0;
}


